import ipaddress
from abc import ABCMeta, abstractmethod
from typing import List


class Host(metaclass=ABCMeta):

    @property
    @abstractmethod
    def image_type(self):
        pass

    @property
    @abstractmethod
    def host_name(self):
        pass

    @property
    @abstractmethod
    def ipv4_address(self):
        pass

    @property
    @abstractmethod
    def ipv4_address_hex(self):
        pass


class DefaultHost(Host):

    def __init__(self, image_type):
        self._image_type = image_type

    @property
    def image_type(self):
        return self._image_type

    @property
    def host_name(self):
        return None

    @property
    def ipv4_address(self):
        return None

    @property
    def ipv4_address_hex(self):
        return "default"


class SpecificHost(Host):

    def __init__(self, name_prefix: str, address: ipaddress.IPv4Address, image_type: str):
        self.name_prefix = name_prefix

        value = address._ip
        first = value % 256
        value //= 256
        second = value % 256
        value //= 256
        third = value % 256
        value //= 256
        fourth = value % 256
        self._address = (fourth, third, second, first)

        self._image_type = image_type

    @property
    def ipv4_address(self) -> str:
        return ".".join((str(x) for x in self._address))

    @property
    def ipv4_address_hex(self) -> str:
        return "".join(('{:02X}'.format(x) for x in self._address))

    @property
    def host_name(self):
        return self.name_prefix + str(self._address[-1])

    @property
    def image_type(self) -> str:
        return self._image_type

    def __str__(self):
        return str(self._address)


class HostGroup(object):

    def __init__(self, name_prefix: str, cidr_string: str):
        self.network = ipaddress.ip_network(cidr_string)
        self.name_prefix = name_prefix
        self._hosts = []

    def add_hosts(self, image_type: str, machine_offset: int, machine_count: int) -> None:
        self._hosts += (SpecificHost(self.name_prefix, x, image_type) for i, x in enumerate(self.network.hosts())
                        if machine_offset <= i + 1 < machine_offset + machine_count)

    def hosts(self) -> List[Host]:
        return self._hosts



# coding=utf-8
import glob

import setuptools

setuptools.setup(
    name="netboot-config",
    version="0.1.8",
    author="Andreas Würl",
    author_email="andreas.wuerl@uniscon.com",
    description="Generator for KIWI based netboot config files",
    long_description_content_type="text/markdown",
    #    install_requires=['configparser==3.5.0','Flask==1.0.2','jsontool==0.2.1','psutil==5.6.1','linux-utils==0.6','pyparsing==2.3.1','requests==2.21.0','rfc5424-logging-handler==1.3.0','Werkzeug==0.14.1', 'netaddr==0.7.19','netifaces==0.10.9'],
    #    url="https://gitlab.uniscon-rnd.de/sealed-platform/sealed-control-plane/tree/develop/SCSIVolumeManagerAPI",
    packages=["netboot_config"],
    scripts=["netboot-config"],
    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        "Programming Language :: Python :: 3.5",

        "License :: OSI Approved :: Apache Software License",

        "Operating System :: POSIX :: Linux",
    ],
)
